'#Language "WWB-COM"
''MACRO TITLE: wHub
' MACRO VERSION: 3.1
'This macro generates a Hub from a selected entity|table
' Dependencies: _wDataVault.bas
'
' Release notes
' 3.1: Rolenames used
' 3.0: Option to generate a Visual Data Lineage
' 2.2: Domains folders updated
' 2.1: Common functions shared with _wDataVault 
' 2.0: Domains added
' 1.0 Initial version
'---------------------------------------------------------------------------
'#Uses "_wDataVault.BAS.hidden"

Option Explicit

#Region "Constants"
Const TITLE$ = "wHub"
Const TIMESTAMPED As Boolean = True
Const MARGIN% = 1
Const NAME_COMPARTMENT As Boolean = True
Const DOMAIN_FOLDER$ = "Hub"

Const DATA_FLOW_NAME$ = "Raw Vault"
Const TRANSFORMATION_HK_NAME$ = "Generate HashKey for"
Const TRANSFORMATION_BK_NAME$ = "Populate Business Key for"

Const HASH_KEY_PREFIX$ = "HK_"
Const HASH_DATATYPE$ = "CHAR"
Const HASH_DATALENGTH% = 32
Const HUB_BGCOLOR& = RGB(192, 192, 255)
Const HUB_FGCOLOR& = RGB(63, 63, 0)
#End Region

#Region "Variables"
Dim aLog$() ' Array of strings for the Logs
Dim laAttributes$() ' Array of strings for the Attributes

Dim MyDictionary As Dictionary
Dim dictionary_list$()

Dim bVDL As Boolean

Dim MyDiagram As Diagram
Dim MyModel As Model
Dim IsLogical As Boolean
Dim MySubModel As SubModel
Dim MyEntity As Entity
Dim MyAttribute As AttributeObj

Dim iLoop%

Dim sParentName$
Dim sChildName$
#End Region

Private Sub Work
#Region "Variables"
	Dim iDictionarySelect%

	Dim MyEntityDisplay As EntityDisplay
	Dim theHubEnt As Entity
	Dim theAttr As AttributeObj

	Dim sHubName$
	Dim sPrefix$
	Dim bUseDomains As Boolean

	Dim MyTransformation As Transformation
	Dim MyTransformationField As TransformationField
	Dim MyTransformationDisplay As TransformationDisplay
	Dim MyDataFlow As DataFlow
	Dim MyLineageComponent As LineageComponent
	Dim MyDataStream As DataStream
	Dim sTransformationName$
#End Region

	LogIt "Work"
	DiagramManager.EnableScreenUpdateEx(False, False)

	DlgEnable("pbStart", False)
	sHubName = DlgText "tbHubName"
	sPrefix = DlgText "tbPrefix"
	bUseDomains = DlgValue "cbDomains"
	bVDL = DlgValue("cbVDL")

	If bUseDomains Then
		iDictionarySelect = DlgValue "dictionary_select"
		If dictionary_list(iDictionarySelect) = "Local" Then
			Set MyDictionary = MyDiagram.Dictionary
		Else
			Set MyDictionary = MyDiagram.EnterpriseDataDictionaries.Item(dictionary_list(iDictionarySelect))
		End If
	End If

	Dim iX%, iY%
	Dim edf%

	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(If(IsLogical, MyEntity.EntityName, MyEntity.TableName))
	If MyEntityDisplay Is Nothing Then
		iX = 0
		iY = 0
	Else
		iX = MyEntityDisplay.HorizontalPosition + MARGIN
		iY = MyEntityDisplay.VerticalPosition   + MARGIN
	End If

	Set theHubEnt = MyModel.Entities.Add(iX, iY)
	theHubEnt.EntityName = LCase(sPrefix & sHubName)
	theHubEnt.TableName = theHubEnt.EntityName
	theHubEnt.Note = "Hub  representing a list of unique business keys"
	theHubEnt.DimModelTableType = "UNDEFINED"

	Set theAttr = theHubEnt.Attributes.Add(LCase(HASH_KEY_PREFIX & theHubEnt.EntityName), True)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, HASH_DATATYPE, HASH_DATALENGTH)

	Dim s%()
	s = DlgValue "mlbAttributes"

    For iLoop = LBound(s) To UBound(s)
		Set MyAttribute = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
		If Not (MyAttribute Is Nothing) Then
			LogIt sChildName & ": " & If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)

			Set theAttr = theHubEnt.Attributes.Add(LCase(If(IsLogical, If(MyAttribute.HasLogicalRoleName,  MyAttribute.LogicalRoleName,  MyAttribute.AttributeName), If(MyAttribute.HasRoleName, MyAttribute.RoleName, MyAttribute.ColumnName))), False)
			theAttr.Datatype = MyAttribute.Datatype
			theAttr.DataLength = MyAttribute.DataLength
			theAttr.DataScale = MyAttribute.DataScale
'			theAttr.NullOption = MyAttribute.NullOption
			theAttr.NullOption = "NOT NULL"
			theAttr.Font.Italic = True
'			theAttr.Color = HUB_FGCOLOR
		End If
    Next

	Set theAttr = theHubEnt.Attributes.Add("dss_record_source", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "VARCHAR", 256)

	Set theAttr = theHubEnt.Attributes.Add("dss_load_date", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "DATETIME", 0)

  	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(theHubEnt.EntityName)

	MyEntityDisplay.BackgroundColor =  HUB_BGCOLOR
	MyEntityDisplay.NonInheritedPrimaryKeysColor = HUB_FGCOLOR
	MyEntityDisplay.NonInheritedNonKeysColor = HUB_FGCOLOR
	MyEntityDisplay.NameColor = If(NAME_COMPARTMENT, HUB_FGCOLOR, HUB_BGCOLOR)
	MyEntityDisplay.DisplayBackgroundColor = True
	MyEntityDisplay.HorizontalPosition = iX
	MyEntityDisplay.VerticalPosition = iY

	If NAME_COMPARTMENT Then
		MySubModel.ShowEntityNameCompartment = NAME_COMPARTMENT

		edf = MySubModel.EntityDisplayFormat
		If (edf <> 7) And (edf <> 8) Then
			MySubModel.EntityDisplayFormat = If(IsLogical, 7, 8)
		End If
	End If

	LogIt "New hub created: " & theHubEnt.EntityName

	Dim l As Line
	Dim ld As LineDisplay

	Set l = MyModel.Lines.Add(1, If(IsLogical, MyEntity.EntityName, MyEntity.TableName), 1, theHubEnt.EntityName)
	Set ld = MySubModel.LineDisplays.Add(l.ID)

	If Not ld Is Nothing Then

		ld.Color = HUB_BGCOLOR

	End If
	

#Region "Visual Data Lineage"
	' Manage Visual Data Lineage
	If bVDL Then
		Set MyDataFlow = MyDiagram.DataFlows.Item(DATA_FLOW_NAME)
	
		If MyDataFlow Is Nothing Then
	
			'data flow doesn't exist so create it
			Set MyDataFlow = MyDiagram.DataFlows.Add(DATA_FLOW_NAME)
	
		End If
	
		If MyDataFlow Is Nothing Then
	
			'if the object is still not initialized something happened when creating it.  log error to log file
			LogIt "Data Flow  <" & DATA_FLOW_NAME & "> could not be created."
			LogIt DiagramManager.GetLastErrorString
	
		Else

			' Add Tranformation for the HashKey
			sTransformationName = TRANSFORMATION_HK_NAME & " " & theHubEnt.EntityName
			'after the data flow is created, create the transformation
			Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
			Set MyTransformation = MyDataFlow.Transformations.Item(sTransformationName)
	
			If MyTransformation Is Nothing And MyTransformationDisplay Is Nothing Then
	
				'if neither the display or object exist add it to the data flow
				Set MyTransformation = MyDataFlow.Transformations.Add(300, 200)
	
				If MyTransformation Is Nothing Then
	
					'log missing transformation in the error string
					LogIt "Transformation  <" & sTransformationName & "> could not be created."
					LogIt DiagramManager.GetLastErrorString
	
				Else
	
					'set the name and display object
					MyTransformation.Name = sTransformationName
					Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
	
					'set the other transformation properties
					MyTransformation.Type = "Calculation"
					MyTransformation.BusinessDefinition = "Calculate HashKey from the Business Keys " & sChildName & "s"
					MyTransformation.CodeDefinition = "INSERT INTO " & theHubEnt.EntityName & " (" & LCase(HASH_KEY_PREFIX & theHubEnt.EntityName) & ")"


					' Add the source
					'see if the lineage component exists in the data flow
					Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, MyEntity.ID)
	
					If MyLineageComponent Is Nothing Then
						' Add the object to the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, MyEntity.ID, 1, 305)
					End If

					' Add data stream between source & transformation
					Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, True)
		
					If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
						'log to error file
						LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
						LogIt DiagramManager.GetLastErrorString
					Else
						
						'now add the input columns
						For iLoop = LBound(s) To UBound(s)
			
							Set theAttr = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
			
							If theAttr Is Nothing Then
			
								'log to error file if column doesn't exist
								LogIt "Model " & sChildName & "  <" & laAttributes(s(iLoop)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
			
							Else
			
								'set input column based on attribute
								Set MyTransformationField = MyTransformation.LineageInputs.Add(5, theAttr.ID)
			
								If MyTransformationField Is Nothing Then
									'log to error file
									LogIt "Transformation Input  <" & laAttributes(s(iLoop)) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								End If
			
							End If	  'transformation existence check
	
					    Next iLoop

					End If ' data stream

					' Add the target
					'see if the lineage component exists in the data flow
					Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, theHubEnt.ID)
	
					If MyLineageComponent Is Nothing Then
						' Add the object to the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, theHubEnt.ID, 650, 300)
					End If

					' Add data stream between source & transformation
					Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, False)
		
					If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
						'log to error file
						LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
						LogIt DiagramManager.GetLastErrorString
					Else
						
						'now add the output columns
						Set theAttr = theHubEnt.Attributes.Item(LCase(HASH_KEY_PREFIX & theHubEnt.EntityName))
		
						If theAttr Is Nothing Then
		
							'log to error file if column doesn't exist
							LogIt "Model " & sChildName & "  <" & LCase(HASH_KEY_PREFIX & theHubEnt.EntityName) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
							LogIt DiagramManager.GetLastErrorString
		
						Else
		
							'set output column based on attribute
							Set MyTransformationField = MyTransformation.LineageOutputs.Add(5, theAttr.ID)
		
							If MyTransformationField Is Nothing Then
								'log to error file
								LogIt "Transformation Output  <" & LCase(HASH_KEY_PREFIX & theHubEnt.EntityName) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
							End If
		
						End If	  'transformation existence check

					End If ' data stream

				End If ' tran is nothing
	
			End If 'tran and tran display check


			' Add Tranformation for the Business Keys
			sTransformationName = TRANSFORMATION_BK_NAME & " " & theHubEnt.EntityName
			' After the data flow is created, create the transformation
			Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
			Set MyTransformation = MyDataFlow.Transformations.Item(sTransformationName)
	
			If MyTransformation Is Nothing And MyTransformationDisplay Is Nothing Then
	
				'if neither the display or object exist add it to the data flow
				Set MyTransformation = MyDataFlow.Transformations.Add(300, 400)
	
				If MyTransformation Is Nothing Then
	
					'log missing transformation in the error string
					LogIt "Transformation  <" & sTransformationName & "> could not be created."
					LogIt DiagramManager.GetLastErrorString
	
				Else
	
					'set the name and display object
					MyTransformation.Name = sTransformationName
					Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
	
					'set the other transformation properties
					MyTransformation.Type = "Select Into"
					MyTransformation.BusinessDefinition = "Copy the Business Keys values"
					MyTransformation.CodeDefinition = "INSERT INTO " & theHubEnt.EntityName & " " & vbCrLf & "SELECT * FROM " & If(IsLogical, MyEntity.EntityName, MyEntity.TableName) & vbCrLf & "WHERE "
	

					' Add the source
					'see if the lineage component exists in the data flow
					Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, MyEntity.ID)
	
					If MyLineageComponent Is Nothing Then
						' Add the object to the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, MyEntity.ID, 100, 100)
					End If

					' Add data stream between source & transformation
					Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, True)
		
					If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
						'log to error file
						LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
						LogIt DiagramManager.GetLastErrorString
					Else
						
						'now add the input columns
						For iLoop = LBound(s) To UBound(s)
			
							Set theAttr = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
			
							If theAttr Is Nothing Then
			
								'log to error file if column doesn't exist
								LogIt "Model " & sChildName & "  <" & laAttributes(s(iLoop)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
			
							Else
			
								'set input column based on attribute
								Set MyTransformationField = MyTransformation.LineageInputs.Add(5, theAttr.ID)
			
								If MyTransformationField Is Nothing Then
									'log to error file
									LogIt "Transformation Input  <" & laAttributes(s(iLoop)) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								End If
			
							End If	  'transformation existence check
	
					    Next iLoop

					End If ' data stream

					' Add the target
					'see if the lineage component exists in the data flow
					Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, theHubEnt.ID)
	
					If MyLineageComponent Is Nothing Then
						' Add the object to the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, theHubEnt.ID, 100, 100)
					End If

					' Add data stream between source & transformation
					Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, False)
		
					If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
						'log to error file
						LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
						LogIt DiagramManager.GetLastErrorString
					Else
						
						'now add the output columns
						For iLoop = LBound(s) To UBound(s)

							Set MyAttribute = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
							If MyAttribute Is Nothing Then
								'log to error file
								LogIt sChildName & "  <" & laAttributes(s(iLoop)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
							Else
								Set theAttr = theHubEnt.Attributes.Item(LCase(If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)))
								If theAttr Is Nothing Then
				
									'log to error file if column doesn't exist
									LogIt "Model " & sChildName & "  <" & LCase(If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
				
								Else
				
									'set output column based on attribute
									Set MyTransformationField = MyTransformation.LineageOutputs.Add(5, theAttr.ID)
				
									If MyTransformationField Is Nothing Then
										'log to error file
										LogIt "Transformation Output  <" & laAttributes(s(iLoop)) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
										LogIt DiagramManager.GetLastErrorString
									End If
				
								End If	  'transformation existence check

							End If

					    Next iLoop

					End If ' data stream

				End If ' tran is nothing
	
			End If 'tran and tran display check

		End If		  'data flow existence check

	End If ' Visual Data Lineage
#End Region

	' Job finished
	MySubModel.ActivateSubModel
	MySubModel.SelectedObjects.Add(1, MyEntity.ID)
	DiagramManager.EnableScreenUpdateEx(True, True)
	DlgText("pbStart", "Completed")
End Sub

Sub Main
	On Error GoTo errHandler

	Dim MySelectedObject As SelectedObject

	InitCommonVars(aLog, MyDiagram, MyModel, MySubModel, IsLogical, sParentName, sChildName)

	ReDim laAttributes(0)

	If MySubModel.SelectedObjects.Count <> 1 Then
		MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select ONE " & sParentName & "!", vbExclamation, TITLE
		Exit Sub
	End If

	For Each MySelectedObject In MySubModel.SelectedObjects
		If MySelectedObject.Type <> 1 Then
			MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select one " & UCase(sParentName) & "!", vbExclamation, TITLE
			Exit Sub
		End If

		Set MyEntity = MyModel.Entities.Item(MySelectedObject.ID)

'		LogIt "Selected " & sParentName & ": " & MyEntity.EntityName

		iLoop = 0
		ReDim laAttributes(0 To MyEntity.Attributes.Count)
		For Each MyAttribute In MyEntity.Attributes
			laAttributes(iLoop) = If(IsLogical, If(MyAttribute.HasLogicalRoleName,  MyAttribute.LogicalRoleName,  MyAttribute.AttributeName), If(MyAttribute.HasRoleName, MyAttribute.RoleName, MyAttribute.ColumnName))
			iLoop = iLoop + 1
		Next MyAttribute

	Next MySelectedObject

	Begin Dialog UserDialog 770,420,TITLE,.DialogFunc ' %GRID:10,7,1,1
		Text 30,7,110,14,"Hub Name:",.hubCaption,1
		TextBox 160,5,600,18,.tbHubName
		Text 30,28,110,14,"Prefix:",.prefixCaption,1
		TextBox 160,26,600,18,.tbPrefix
		GroupBox 10,49,750,119,"Attributes",.gbAttributes
		MultiListBox 20,63,730,98,laAttributes(),.mlbAttributes,1
		CheckBox 20,175,730,14,"Domains for common Data Vault attributes/columns",.cbDomains
		Text 20,196,120,14,"Select Dictionary: ",.tDictionary,1
		DropListBox 160,193,590,112,dictionary_list(),.dictionary_select
		CheckBox 20,217,730,14,"Generate Visual Data Lineage",.cbVDL
		PushButton 350,238,90,21,"Start",.pbStart
		ListBox 20,266,740,112,aLog(),.lbLog,1
		PushButton 20,392,90,21,"Blog post",.pbBlog
		PushButton 350,392,90,21,"Close",.cbCancel
		CancelButton 355,695,80,15 ' out of screen: allow to close the dialog with the X
	End Dialog

	Dim dlg As UserDialog

	init_dictionary_list(dictionary_list, MyDiagram, MyDictionary)

	dlg.tbHubName = LCase(If(IsLogical, MyEntity.EntityName, MyEntity.TableName))
	dlg.tbPrefix = "h_"
	dlg.cbDomains = True
	dlg.cbVDL = True
	dlg.dictionary_select = UBound(dictionary_list) 'Default DD = Latest one: if only one, it's the local one, otherwise, an EDD

	Dialog dlg

	Exit Sub

	errHandler:
		ManageError(Err, TITLE, TIMESTAMPED)
End Sub

Private Function DialogFunc(DlgItem$, Action%, SuppValue&) As Boolean
	Dim i%
	Dim iLoop%
	Dim s%()
    Select Case Action%
		Case 1 ' Dialog box initialization

			InitDialog("tbHubName", MyDiagram, MyModel, MySubModel, aLog, TIMESTAMPED)

			DlgText("gbAttributes", "Select Business Key(s) " & sChildName & "s")
			DlgText("cbDomains", "Domains for common Data Vault " & If(IsLogical, "Attributes", "Columns"))

        Case 2        ' Value changing or button pressed
		
			DialogFunc = DialogControlUpdated("tbHubName", DlgItem)
		    If DlgItem = "pbStart" Then
				Work
		    End If

		Case 3 ' TextBox or ComboBox text changed
		
			RefreshButtonStart("tbHubName")

    End Select
End Function

Private Sub LogIt(ByVal txt As String)
	Log(txt, aLog, TIMESTAMPED)
End Sub
