'#Language "WWB-COM"
''MACRO TITLE: wLink
' MACRO VERSION: 3.1
'This macro generates a Link from selected hubs and|or satellites
' Pre-requisites: You need to select the Hubs (2+) before you run the macro.
' Dependencies: _wDataVault.bas
'
' Release notes
' 3.1: Satellites can be used to create a Link
' 3.0: Option to generate a Visual Data Lineage
' 2.2: Domains folders updated
' 2.1: Common functions shared with _wDataVault 
' 2.0: Domains added
' 1.0 Initial version
'---------------------------------------------------------------------------
'#Uses "_wDataVault.BAS.hidden"

Option Explicit

#Region "Constants"
Const TITLE As String = "wLink"
Const TIMESTAMPED As Boolean = True
Const MARGIN% = 1
Const NAME_COMPARTMENT As Boolean = True
Const DOMAIN_FOLDER$ = "Link"

Const DATA_FLOW_NAME$ = "Raw Vault"
Const TRANSFORMATION_NAME$ = "Populate "

Const HASH_KEY_PREFIX$ = "HK_"
Const HASH_DATATYPE$ = "CHAR"
Const HASH_DATALENGTH% = 32
'Const HUB_PREFIX$ = "h_"
Const LINK_BGCOLOR& = RGB(255, 192, 192)
Const LINK_FGCOLOR& = RGB(0, 63, 63)
#End Region

#Region "Variables"
Dim aLog$() ' Array of strings for the Logs
Dim laHub() As Entity ' Array of Entity for the Hubs
Dim laHubName$() ' Array of strings for the Hubs names

Dim MyDictionary As Dictionary
Dim dictionary_list$()

Dim bVDL As Boolean

Dim MyDiagram As Diagram
Dim MyModel As Model
Dim IsLogical As Boolean
Dim MySubModel As SubModel
Dim MySelectedObject As SelectedObject

Dim iLoop%

Dim sParentName$, sParentsName$
Dim sChildName$
#End Region

Private Sub Work
#Region "Variables"
	Dim iDictionarySelect%

	Dim MyEntityDisplay As EntityDisplay
	Dim theLinkEnt As Entity
	Dim theAttr As AttributeObj

	Dim sLinkName$
	Dim sPrefix$
	Dim bUseDomains As Boolean

	Dim MyTransformation As Transformation
	Dim MyTransformationField As TransformationField
	Dim MyTransformationDisplay As TransformationDisplay
	Dim MyDataFlow As DataFlow
	Dim MyLineageComponent As LineageComponent
	Dim MyDataStream As DataStream
	Dim sTransformationName$
#End Region

	LogIt "Work"
	DiagramManager.EnableScreenUpdateEx(False, False)

	DlgEnable("pbStart", False)
	sLinkName = DlgText "tbLinkName"
	sPrefix = DlgText "tbPrefix"
	bUseDomains = DlgValue "cbDomains"
	bVDL = DlgValue("cbVDL")

	If bUseDomains Then
		iDictionarySelect = DlgValue "dictionary_select"
		If dictionary_list(iDictionarySelect) = "Local" Then
			Set MyDictionary = MyDiagram.Dictionary
		Else
			Set MyDictionary = MyDiagram.EnterpriseDataDictionaries.Item(dictionary_list(iDictionarySelect))
		End If
	End If

	Dim iX%, iY%
	Dim edf%
	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(If(IsLogical, laHub(LBound(laHub)).EntityName, laHub(LBound(laHub)).TableName))
	If MyEntityDisplay Is Nothing Then
		iX = 0
		iY = 0
	Else
		iX = MyEntityDisplay.HorizontalPosition + MARGIN
		iY = MyEntityDisplay.VerticalPosition   + MARGIN
	End If

	Set theLinkEnt = MyModel.Entities.Add(iX, iY)
	theLinkEnt.EntityName = LCase(sPrefix & sLinkName)
	theLinkEnt.TableName = theLinkEnt.EntityName
	theLinkEnt.Note = "Link  describes a unique list of relationships/interactions between business keys"

	Set theAttr = theLinkEnt.Attributes.Add(LCase(HASH_KEY_PREFIX & theLinkEnt.EntityName), True)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, HASH_DATATYPE, HASH_DATALENGTH)

	Dim rLink As Relationship

    For iLoop = LBound(laHub) To UBound(laHub)
		Set rLink = MyModel.Relationships.Add(If(IsLogical, laHub(iLoop).EntityName, laHub(iLoop).TableName), theLinkEnt.EntityName, 1)
    Next iLoop

	Set theAttr = theLinkEnt.Attributes.Add("dss_record_source", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "VARCHAR", 256)

	Set theAttr = theLinkEnt.Attributes.Add("dss_load_date", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "DATETIME", 0)

  	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(theLinkEnt.EntityName)

	MyEntityDisplay.BackgroundColor =  LINK_BGCOLOR
	MyEntityDisplay.NonInheritedPrimaryKeysColor = LINK_FGCOLOR
	MyEntityDisplay.NonInheritedNonKeysColor = LINK_FGCOLOR
	MyEntityDisplay.InheritedNonKeysColor = LINK_FGCOLOR
	MyEntityDisplay.NameColor = If(NAME_COMPARTMENT, LINK_FGCOLOR, LINK_BGCOLOR)
	MyEntityDisplay.DisplayBackgroundColor = True
	MyEntityDisplay.HorizontalPosition = iX
	MyEntityDisplay.VerticalPosition = iY

	If NAME_COMPARTMENT Then
		MySubModel.ShowEntityNameCompartment = NAME_COMPARTMENT

		edf = MySubModel.EntityDisplayFormat
		If (edf <> 7) And (edf <> 8) Then
			MySubModel.EntityDisplayFormat = If(IsLogical, 7, 8)
		End If
	End If

	LogIt "New link created: " & theLinkEnt.EntityName

#Region "Visual Data Lineage"
	' Manage Visual Data Lineage
	If bVDL Then
		
		Set MyDataFlow = MyDiagram.DataFlows.Item(DATA_FLOW_NAME)
	
		If MyDataFlow Is Nothing Then
	
			'data flow doesn't exist so create it
			Set MyDataFlow = MyDiagram.DataFlows.Add(DATA_FLOW_NAME)
	
		End If
	
		If MyDataFlow Is Nothing Then
	
			'if the object is still not initialized something happened when creating it.  log error to log file
			LogIt "Data Flow  <" & DATA_FLOW_NAME & "> could not be created."
			LogIt DiagramManager.GetLastErrorString
	
		Else

			' Add Tranformation for the HashKey
			sTransformationName = TRANSFORMATION_NAME & " " & theLinkEnt.EntityName
			'after the data flow is created, create the transformation
			Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
			Set MyTransformation = MyDataFlow.Transformations.Item(sTransformationName)
	
			If MyTransformation Is Nothing And MyTransformationDisplay Is Nothing Then
	
				'if neither the display or object exist add it to the data flow
				Set MyTransformation = MyDataFlow.Transformations.Add(300, 600)
	
				If MyTransformation Is Nothing Then
	
					'log missing transformation in the error string
					LogIt "Transformation  <" & sTransformationName & "> could not be created."
					LogIt DiagramManager.GetLastErrorString
	
				Else
	
					'set the name and display object
					MyTransformation.Name = sTransformationName
					Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
	
					'set the other transformation properties
					MyTransformation.Type = "Direct Map"
					MyTransformation.BusinessDefinition = "HashKeys from the Hubs"
					MyTransformation.CodeDefinition = ""

					' Add the sources
				    For iLoop = LBound(laHub) To UBound(laHub)
						
						'see if the lineage component exists in the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, laHub(iLoop).ID)
		
						If MyLineageComponent Is Nothing Then
							' Add the object to the data flow
							Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, laHub(iLoop).ID, 650, (400 + (200 * iLoop)))
						End If
	
						' Add data stream between source & transformation
						Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, True)
	
						If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
							'log to error file
							LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
							LogIt DiagramManager.GetLastErrorString
						Else
							
							'now add the input columns
							For Each theAttr In laHub(iLoop).Attributes

								' get the hashkey
								If theAttr.PrimaryKey And (InStr(LCase(theAttr.AttributeName),  LCase(HASH_KEY_PREFIX)) = 1) Then

									'set input column based on attribute
									Set MyTransformationField = MyTransformation.LineageInputs.Add(5, theAttr.ID)
				
									If MyTransformationField Is Nothing Then

										'log to error file
										LogIt "Transformation Output  <" & LCase(HASH_KEY_PREFIX & laHub(iLoop).EntityName) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
										LogIt DiagramManager.GetLastErrorString

									End If

									' HK found, exit the loop
									Exit For

								End If

							Next theAttr

						End If ' data stream

						'						Set rLink = MyModel.Relationships.Add(If(IsLogical, laHub(iLoop).EntityName, laHub(iLoop).TableName), theLinkEnt.EntityName, 1)
				    Next iLoop

					' Add the target
					'see if the lineage component exists in the data flow
					Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, theLinkEnt.ID)
	
					If MyLineageComponent Is Nothing Then
						' Add the object to the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, theLinkEnt.ID, 650, 600)
					End If

					' Add data stream between source & transformation
					Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, False)
		
					If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
						'log to error file
						LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
						LogIt DiagramManager.GetLastErrorString
					Else
						
						'now add the output columns
						For Each theAttr In theLinkEnt.Attributes

							' get the hashkey
							If theAttr.ForeignKey And (InStr(LCase(theAttr.AttributeName),  LCase(HASH_KEY_PREFIX)) = 1) Then

								'set output column based on attribute
								Set MyTransformationField = MyTransformation.LineageOutputs.Add(5, theAttr.ID)
			
								If MyTransformationField Is Nothing Then
									'log to error file
									LogIt "Transformation Output  <" & LCase(HASH_KEY_PREFIX & theLinkEnt.EntityName) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								End If

							End If

						Next theAttr

					End If ' data stream

				End If ' tran is nothing
	
			End If 'tran and tran display check

		End If		  'data flow existence check

	End If ' Visual Data Lineage
#End Region

	' Job finished
	MySubModel.ActivateSubModel
	For iLoop = LBound(laHub) To UBound(laHub)
		MySubModel.SelectedObjects.Add(1, laHub(iLoop).ID)
	Next
	DiagramManager.EnableScreenUpdateEx(True, True)
	DlgText("pbStart", "Completed")
End Sub

Sub Main
	On Error GoTo errHandler

	Dim sName$

	InitCommonVars(aLog, MyDiagram, MyModel, MySubModel, IsLogical, sParentName, sChildName)

	sParentsName = If(IsLogical, "Entities", "Tables")

	If MySubModel.SelectedObjects.Count < 2 Then
		MsgBox "Error:" & vbCrLf & vbCrLf & "You must select at least 2 hubs!", vbExclamation, TITLE
		Exit Sub
	End If

	ReDim laHub(0 To MySubModel.SelectedObjects.Count - 1)
	ReDim laHubName(0 To MySubModel.SelectedObjects.Count - 1)
	iLoop = 0

	For Each MySelectedObject In MySubModel.SelectedObjects
		If (MySelectedObject.Type <> 1) Then
			MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select " & sParentsName & "!", vbExclamation, TITLE
			Exit Sub
		End If

		Set laHub(iLoop) = MyModel.Entities.Item(MySelectedObject.ID)
		laHubName(iLoop) = If(IsLogical, laHub(iLoop).EntityName, laHub(iLoop).TableName)

'		If (LCase(Left(If(IsLogical, laHub(iLoop).EntityName, laHub(iLoop).TableName), 2)) <> LCase(HUB_PREFIX)) Then
'			MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select hubs!", vbExclamation, TITLE
'			Exit Sub
'		End If

'		Debug.Print "Selected Hub: " & If(IsLogical, laHub(iLoop).EntityName, laHub(iLoop).TableName)

		iLoop = iLoop + 1
	Next MySelectedObject

	Begin Dialog UserDialog 770,420,TITLE,.DialogFunc ' %GRID:10,7,1,1
		Text 30,7,110,14,"Link Name:",.linkCaption,1
		TextBox 160,5,600,18,.tbLinkName
		Text 30,28,110,14,"Prefix:",.prefixCaption,1
		TextBox 160,26,600,18,.tbPrefix
		GroupBox 10,49,750,119,"Selected hubs",.gbAttributes
		MultiListBox 20,63,730,98,laHubName(),.mlbAttributes,3
		CheckBox 20,175,730,14,"Domains for common Data Vault attributes/columns",.cbDomains
		Text 20,196,120,14,"Select Dictionary: ",.tDictionary,1
		DropListBox 160,193,590,112,dictionary_list(),.dictionary_select
		CheckBox 20,217,730,14,"Generate Visual Data Lineage",.cbVDL
		PushButton 350,238,90,21,"Start",.pbStart
		ListBox 20,266,740,112,aLog(),.lbLog,1
		PushButton 20,392,90,21,"Blog post",.pbBlog
		PushButton 350,392,90,21,"Close",.cbCancel
		CancelButton 355,695,80,15 ' out of screen: allow to close the dialog with the X
	End Dialog

	Dim dlg As UserDialog

	sName = ""

	For iLoop = 0 To UBound(laHubName)
		sName = sName & "_" & Right(laHubName(iLoop), Len(laHubName(iLoop)) - 2)
	Next iLoop
	sName = Right(sName, Len(sName) - 1)

	init_dictionary_list(dictionary_list, MyDiagram, MyDictionary)

	dlg.tbLinkName = LCase(sName)
	dlg.tbPrefix = "l_"
	dlg.cbDomains = True
	dlg.cbVDL = True
	dlg.dictionary_select = UBound(dictionary_list) 'Default DD = Latest one: if only one, it's the local one, otherwise, an EDD

	Dialog dlg

	Exit Sub

	errHandler:
		ManageError(Err, TITLE, TIMESTAMPED)
End Sub

Private Function DialogFunc(DlgItem$, Action%, SuppValue&) As Boolean
	Dim i%, iLoop%, s%()

    Select Case Action%
		Case 1 ' Dialog box initialization
		
			InitDialog("tbLinkName", MyDiagram, MyModel, MySubModel, aLog, TIMESTAMPED)
			DlgEnable("mlbAttributes", False)

        Case 2        ' Value changing or button pressed
		
			DialogFunc = DialogControlUpdated("tbLinkName", DlgItem)
		    If DlgItem = "pbStart" Then
				Work
		    End If

		Case 3 ' TextBox or ComboBox text changed
		
			RefreshButtonStart("tbLinkName")

    End Select
End Function

Private Sub LogIt(ByVal txt As String)
	Log(txt, aLog, TIMESTAMPED)
End Sub
