'#Language "WWB-COM"
''MACRO TITLE: wSatellite
' MACRO VERSION: 3.1
'This macro generates a Satellite from a selected entity|table
' Pre-requisites: To generate the relationship between the Hub & the
'    Satellite, you need to run the wHub macro first.
' Dependencies: _wDataVault.bas
'
' Release notes
' 3.1: Rolenames used
' 3.0: Option to generate a Visual Data Lineage
' 2.2: Domains folders updated
' 2.1: Common functions shared with _wDataVault 
' 2.0: Domains added
' 1.0 Initial version
'---------------------------------------------------------------------------
'#Uses "_wDataVault.BAS.hidden"

Option Explicit

#Region "Constants"
Const TITLE As String = "wSatellite"
Const TIMESTAMPED As Boolean = True
Const MARGIN% = 1
Const NAME_COMPARTMENT As Boolean = True
Const DOMAIN_FOLDER$ = "Satellite"

Const DATA_FLOW_NAME$ = "Raw Vault"
Const TRANSFORMATION_NAME$ = "Populate"

Const HASH_KEY_PREFIX$ = "HK_"
Const HASH_DATATYPE$ = "CHAR"
Const HASH_DATALENGTH% = 32
Const SAT_BGCOLOR& = RGB(255, 255, 192)
Const SAT_FGCOLOR& = RGB(0, 0, 63)
#End Region

#Region "Variables"
Dim aLog$() ' Array of strings for the Logs
Dim laAttributes$() ' Array of strings for the Attributes

Dim MyDictionary As Dictionary
Dim dictionary_list$()

Dim bVDL As Boolean

Dim MyDiagram As Diagram
Dim MyModel As Model
Dim IsLogical As Boolean
Dim MyEntity As Entity
Dim MySubModel As SubModel
Dim MyAttribute As AttributeObj
Dim MySelectedObject As SelectedObject

Dim iLoop%

Dim sParentName$
Dim sChildName$
#End Region

Private Sub Work
#Region "Variables"
	Dim iDictionarySelect%

	Dim MyEntityDisplay As EntityDisplay
	Dim theSatelliteEnt As Entity
	Dim theAttr As AttributeObj

	Dim sSatelliteName$
	Dim sPrefix$, sSuffix$
	Dim bUseDomains As Boolean

	Dim MyTransformation As Transformation
	Dim MyTransformationField As TransformationField
	Dim MyTransformationDisplay As TransformationDisplay
	Dim MyDataFlow As DataFlow
	Dim MyLineageComponent As LineageComponent
	Dim MyDataStream As DataStream
	Dim sTransformationName$

	Dim eParentHub As Entity
	Dim rHubSat As Relationship

	Dim iX%, iY%
	Dim edf%

	Dim s%()
#End Region

	LogIt "Work"
	DiagramManager.EnableScreenUpdateEx(False, False)

	DlgEnable("pbStart", False)
	sSatelliteName = DlgText "tbSatelliteName"
	sPrefix = DlgText "tbPrefix"
	sSuffix = DlgText "tbSuffix"
	bUseDomains = DlgValue "cbDomains"
	bVDL = DlgValue("cbVDL")

	If bUseDomains Then
		iDictionarySelect = DlgValue "dictionary_select"
		If dictionary_list(iDictionarySelect) = "Local" Then
			Set MyDictionary = MyDiagram.Dictionary
		Else
			Set MyDictionary = MyDiagram.EnterpriseDataDictionaries.Item(dictionary_list(iDictionarySelect))
		End If
	End If

	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(If(IsLogical, MyEntity.EntityName, MyEntity.TableName))
	If MyEntityDisplay Is Nothing Then
		iX = 0
		iY = 0
	Else
		iX = MyEntityDisplay.HorizontalPosition + MARGIN
		iY = MyEntityDisplay.VerticalPosition   + MARGIN
	End If

	Set theSatelliteEnt = MyModel.Entities.Add(iX, iY)
	theSatelliteEnt.EntityName = LCase(sPrefix & sSatelliteName & sSuffix)
	theSatelliteEnt.TableName = theSatelliteEnt.EntityName
	theSatelliteEnt.Note = "Satellites  contain descriptions and the contexts of the business keys or links"
	theSatelliteEnt.DimModelTableType = "UNDEFINED"

	Set eParentHub = MyModel.Entities(LCase("h_" & sSatelliteName))
	If eParentHub Is Nothing Then
		LogIt "Hub has not been found. You should use the wHub macro before using this one."
		Set theAttr = theSatelliteEnt.Attributes.Add(LCase(HASH_KEY_PREFIX & "h_" & sSatelliteName), True)
		UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, HASH_DATATYPE, HASH_DATALENGTH)
	Else
		Set rHubSat = MyModel.Relationships.Add(eParentHub.EntityName, theSatelliteEnt.EntityName, 0)
	End If

	s = DlgValue "mlbAttributes"

    For iLoop = LBound(s) To UBound(s)
		Set MyAttribute = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
		If Not (MyAttribute Is Nothing) Then
			LogIt sChildName & ": " & If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)

			Set theAttr = theSatelliteEnt.Attributes.Add(LCase(If(IsLogical, If(MyAttribute.HasLogicalRoleName,  MyAttribute.LogicalRoleName,  MyAttribute.AttributeName), If(MyAttribute.HasRoleName, MyAttribute.RoleName, MyAttribute.ColumnName))), False)
			theAttr.Datatype = MyAttribute.Datatype
			theAttr.DataLength = MyAttribute.DataLength
			theAttr.DataScale = MyAttribute.DataScale
			theAttr.NullOption = "NOT NULL"
'			theAttr.NullOption = MyAttribute.NullOption
			theAttr.Font.Italic = True
'			theAttr.Color = SAT_FGCOLOR
		End If
    Next

	Set theAttr = theSatelliteEnt.Attributes.Add("dss_change_hash", False)
	theAttr.Datatype = HASH_DATATYPE
	theAttr.DataLength = HASH_DATALENGTH
'	theAttr.Color = SAT_FGCOLOR

	Set theAttr = theSatelliteEnt.Attributes.Add("dss_record_source", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "VARCHAR", 256)

	Set theAttr = theSatelliteEnt.Attributes.Add("dss_load_date", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "DATETIME", 0)

	Set theAttr = theSatelliteEnt.Attributes.Add("dss_start_date", True)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "DATETIME", 0)

	Set theAttr = theSatelliteEnt.Attributes.Add("dss_version", False)
	UseDomain(bUseDomains, MyDictionary, DOMAIN_FOLDER, theAttr, "INTEGER", 0)

  	Set MyEntityDisplay = MySubModel.EntityDisplays.Item(theSatelliteEnt.EntityName)

'	DiagramManager.EnableScreenUpdateEx(False, False)

	MyEntityDisplay.BackgroundColor = SAT_BGCOLOR
	MyEntityDisplay.InheritedPrimaryKeysColor = SAT_FGCOLOR
	MyEntityDisplay.NonInheritedPrimaryKeysColor = SAT_FGCOLOR
	MyEntityDisplay.NonInheritedNonKeysColor = SAT_FGCOLOR
	MyEntityDisplay.NameColor = If(NAME_COMPARTMENT, SAT_FGCOLOR, SAT_BGCOLOR)
	MyEntityDisplay.DisplayBackgroundColor = True
	MyEntityDisplay.HorizontalPosition = iX
	MyEntityDisplay.VerticalPosition = iY

	If NAME_COMPARTMENT Then
		MySubModel.ShowEntityNameCompartment = NAME_COMPARTMENT

		edf = MySubModel.EntityDisplayFormat
		If (edf <> 7) And (edf <> 8) Then
			MySubModel.EntityDisplayFormat = If(IsLogical, 7, 8)
		End If
	End If

'	DiagramManager.EnableScreenUpdateEx(True, True)

	LogIt "New satellite created: " & theSatelliteEnt.EntityName

	Dim l As Line
	Dim ld As LineDisplay

	Set l = MyModel.Lines.Add(1, If(IsLogical, MyEntity.EntityName, MyEntity.TableName), 1, theSatelliteEnt.EntityName)
	Set ld = MySubModel.LineDisplays.Add(l.ID)

	If Not ld Is Nothing Then
		
		ld.Color = SAT_BGCOLOR

	End If
	

#Region "Visual Data Lineage"
	' Manage Visual Data Lineage
	If bVDL Then
		
		If eParentHub Is Nothing Then

			LogIt "Visual Data Lineage not generated without the Hub"

		Else

			Set MyDataFlow = MyDiagram.DataFlows.Item(DATA_FLOW_NAME)
		
			If MyDataFlow Is Nothing Then
		
				'data flow doesn't exist so create it
				Set MyDataFlow = MyDiagram.DataFlows.Add(DATA_FLOW_NAME)
		
			End If
		
			If MyDataFlow Is Nothing Then
		
				'if the object is still not initialized something happened when creating it.  log error to log file
				LogIt "Data Flow  <" & DATA_FLOW_NAME & "> could not be created."
				LogIt DiagramManager.GetLastErrorString
		
			Else
	
				' Add Tranformation for the HashKey
				sTransformationName = TRANSFORMATION_NAME & " " & theSatelliteEnt.EntityName
				'after the data flow is created, create the transformation
				Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
				Set MyTransformation = MyDataFlow.Transformations.Item(sTransformationName)
		
				If MyTransformation Is Nothing And MyTransformationDisplay Is Nothing Then
		
					'if neither the display or object exist add it to the data flow
					Set MyTransformation = MyDataFlow.Transformations.Add(900, 200)
		
					If MyTransformation Is Nothing Then
		
						'log missing transformation in the error string
						LogIt "Transformation  <" & sTransformationName & "> could not be created."
						LogIt DiagramManager.GetLastErrorString
		
					Else
		
						'set the name and display object
						MyTransformation.Name = sTransformationName
						Set MyTransformationDisplay = MyDataFlow.TransformationDisplays.Item(sTransformationName)
		
						'set the other transformation properties
						MyTransformation.Type = "Select Into"
						MyTransformation.BusinessDefinition = "Copy descriptions and contexts"
						MyTransformation.CodeDefinition = "INSERT INTO " & theSatelliteEnt.EntityName & " (" & LCase(HASH_KEY_PREFIX & eParentHub.EntityName) & ", ...)"
	
	
						' Add the 1st source
						'see if the lineage component exists in the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, eParentHub.ID)
		
						If MyLineageComponent Is Nothing Then
							' Add the object to the data flow
							Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, eParentHub.ID, 650, 300)
						End If
	
						' Add data stream between source & transformation
						Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, True)
			
						If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
							'log to error file
							LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
							LogIt DiagramManager.GetLastErrorString
						Else
	
							'now add the input column
							Set theAttr = eParentHub.Attributes.Item(LCase(HASH_KEY_PREFIX & eParentHub.EntityName))
			
							If theAttr Is Nothing Then
			
								'log to error file if column doesn't exist
								LogIt "Model " & sChildName & "  <" & LCase(HASH_KEY_PREFIX & eParentHub.EntityName) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
			
							Else
			
								'set input column based on attribute
								Set MyTransformationField = MyTransformation.LineageInputs.Add(5, theAttr.ID)
			
								If MyTransformationField Is Nothing Then
									'log to error file
									LogIt "Transformation Output  <" & LCase(HASH_KEY_PREFIX & eParentHub.EntityName) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								End If
	
							End If
	
						End If ' data stream
	
						' Add the 2nd source
						'see if the lineage component exists in the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, MyEntity.ID)
		
						If MyLineageComponent Is Nothing Then
							' Add the object to the data flow
							Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, MyEntity.ID, 0, 300)
						End If
	
						' Add data stream between source & transformation
						Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, True)
			
						If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
							'log to error file
							LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
							LogIt DiagramManager.GetLastErrorString
						Else
	
							'now add the input columns
							For iLoop = LBound(s) To UBound(s)
				
								Set theAttr = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
				
								If theAttr Is Nothing Then
				
									'log to error file if column doesn't exist
									LogIt "Model " & sChildName & "  <" & laAttributes(s(iLoop)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
				
								Else
				
									'set input column based on attribute
									Set MyTransformationField = MyTransformation.LineageInputs.Add(5, theAttr.ID)
				
									If MyTransformationField Is Nothing Then
										'log to error file
										LogIt "Transformation Input  <" & laAttributes(s(iLoop)) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
										LogIt DiagramManager.GetLastErrorString
									End If
				
								End If	  'transformation existence check
		
						    Next iLoop
	
						End If ' data stream
	
						' Add the target
						'see if the lineage component exists in the data flow
						Set MyLineageComponent = MyDataFlow.LineageComponents.ItemBySource(1, theSatelliteEnt.ID)
	
						If MyLineageComponent Is Nothing Then
							' Add the object to the data flow
							Set MyLineageComponent = MyDataFlow.LineageComponents.Add(1, theSatelliteEnt.ID, 1300, 300)
						End If
	
						' Add data stream between source & transformation
						Set MyDataStream = MyDataFlow.DataStreams.Add(MyTransformation.Name, MyLineageComponent.ID, False)
			
						If MyDataStream Is Nothing And Not(MyLineageComponent Is Nothing) Then
							'log to error file
							LogIt "Data Stream between  <" & MyLineageComponent.Name & "> and <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
							LogIt DiagramManager.GetLastErrorString
						Else
							
							'now add the output column for hashkey
							Set theAttr = theSatelliteEnt.Attributes.Item(LCase(HASH_KEY_PREFIX & eParentHub.EntityName))
	
							If theAttr Is Nothing Then
			
								'log to error file if column doesn't exist
								LogIt "Model " & sChildName & "  <" & LCase(HASH_KEY_PREFIX & eParentHub.EntityName) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
								LogIt DiagramManager.GetLastErrorString
			
							Else
			
								'set output column based on attribute
								Set MyTransformationField = MyTransformation.LineageOutputs.Add(5, theAttr.ID)
			
								If MyTransformationField Is Nothing Then
									'log to error file
									LogIt "Transformation Output  <" & LCase(HASH_KEY_PREFIX & eParentHub.EntityName) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								End If
			
							End If	  'transformation existence check
	
							' Add the other columns
	
							'now add the output columns
							For iLoop = LBound(s) To UBound(s)
	
								Set MyAttribute = MyEntity.Attributes.Item(laAttributes(s(iLoop)))
								If MyAttribute Is Nothing Then
									'log to error file
									LogIt sChildName & "  <" & laAttributes(s(iLoop)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
									LogIt DiagramManager.GetLastErrorString
								Else
									Set theAttr = theSatelliteEnt.Attributes.Item(LCase(If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)))
									If theAttr Is Nothing Then
					
										'log to error file if column doesn't exist
										LogIt "Model " & sChildName & "  <" & LCase(If(IsLogical, MyAttribute.AttributeName, MyAttribute.ColumnName)) & "> could not be found in data flow <" & MyDataFlow.Name & ">."
										LogIt DiagramManager.GetLastErrorString
					
									Else
					
										'set output column based on attribute
										Set MyTransformationField = MyTransformation.LineageOutputs.Add(5, theAttr.ID)
					
										If MyTransformationField Is Nothing Then
											'log to error file
											LogIt "Transformation Output  <" & laAttributes(s(iLoop)) & "> between Lineage Component <" & MyLineageComponent.Name & "> and Transformation <" & MyTransformation.Name & "> could not be added in data flow <" & MyDataFlow.Name & ">."
											LogIt DiagramManager.GetLastErrorString
										End If
					
									End If	  'transformation existence check
	
								End If
	
						    Next iLoop
	
						End If ' data stream
	
					End If ' tran is nothing
		
				End If 'tran and tran display check
	
			End If		  'data flow existence check

		End If ' Hub created before using this macro

	End If ' Visual Data Lineage
#End Region

	' Job finished
	MySubModel.ActivateSubModel
	MySubModel.SelectedObjects.Add(1, MyEntity.ID)
	MySubModel.ActivateSubModel
	DiagramManager.EnableScreenUpdateEx(True, True)
End Sub

Sub Main
	On Error GoTo errHandler

	InitCommonVars(aLog, MyDiagram, MyModel, MySubModel, IsLogical, sParentName, sChildName)

	ReDim laAttributes(0)

	If MySubModel.SelectedObjects.Count <> 1 Then
		MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select ONE " & sParentName & "!", vbExclamation, TITLE
		Exit Sub
	End If

	For Each MySelectedObject In MySubModel.SelectedObjects
		If MySelectedObject.Type <> 1 Then
			MsgBox "Error:" & vbCrLf & vbCrLf & "You can only select one " & UCase(sParentName) & "!", vbExclamation, TITLE
			Exit Sub
		End If

		Set MyEntity = MyModel.Entities.Item(MySelectedObject.ID)

'		LogIt "Selected " & sParentName & ": " & MyEntity.EntityName

		iLoop = 0
		ReDim laAttributes(0 To MyEntity.Attributes.Count)
		For Each MyAttribute In MyEntity.Attributes
			laAttributes(iLoop) = If(IsLogical, If(MyAttribute.HasLogicalRoleName,  MyAttribute.LogicalRoleName,  MyAttribute.AttributeName), If(MyAttribute.HasRoleName, MyAttribute.RoleName, MyAttribute.ColumnName))
			iLoop = iLoop + 1
		Next MyAttribute

	Next MySelectedObject

	Begin Dialog UserDialog 770,420,TITLE,.DialogFunc ' %GRID:10,7,1,1
		Text 30,7,110,14,"Satellite Name:",.satelliteCaption,1
		TextBox 160,5,600,18,.tbSatelliteName
		Text 30,28,110,14,"Prefix:",.prefixCaption,1
		TextBox 160,26,600,18,.tbPrefix
		Text 30,49,110,14,"Suffix:",.suffixCaption,1
		TextBox 160,47,600,18,.tbSuffix
		GroupBox 10,77,750,119,"Attributes",.gbAttributes
		MultiListBox 20,91,730,98,laAttributes(),.mlbAttributes,1
		CheckBox 20,203,730,14,"Domains for common Data Vault attributes/columns",.cbDomains
		Text 20,224,120,14,"Select Dictionary: ",.tDictionary,1
		DropListBox 160,221,590,112,dictionary_list(),.dictionary_select
		CheckBox 20,245,730,14,"Generate Visual Data Lineage",.cbVDL
		PushButton 350,273,90,21,"Start",.pbStart
		ListBox 20,301,740,77,aLog(),.lbLog,1
		PushButton 20,392,90,21,"Blog post",.pbBlog
		PushButton 350,392,90,21,"Close",.cbCancel
		CancelButton 355,695,80,15 ' out of screen: allow to close the dialog with the X
	End Dialog

	Dim dlg As UserDialog

	init_dictionary_list(dictionary_list, MyDiagram, MyDictionary)

	dlg.tbSatelliteName = LCase(If(IsLogical, MyEntity.EntityName, MyEntity.TableName))
	dlg.tbPrefix = "s_"
	dlg.tbSuffix = "_lroc"
	dlg.cbDomains = True
	dlg.cbVDL = True
	dlg.dictionary_select = UBound(dictionary_list) 'Default DD = Latest one: if only one, it's the local one, otherwise, an EDD

	Dialog dlg


	Exit Sub

	errHandler:
		ManageError(Err, TITLE, TIMESTAMPED)
End Sub

Private Function DialogFunc(DlgItem$, Action%, SuppValue&) As Boolean
	Dim i%, iLoop%, s%()

    Select Case Action%
		Case 1 ' Dialog box initialization

			InitDialog("tbSatelliteName", MyDiagram, MyModel, MySubModel, aLog, TIMESTAMPED)

			DlgText("gbAttributes", "Select Changing " & sChildName & "s")

        Case 2        ' Value changing or button pressed
		
			DialogFunc = DialogControlUpdated("tbSatelliteName", DlgItem)
		    If DlgItem = "pbStart" Then
				Work
		    End If

		Case 3 ' TextBox or ComboBox text changed
		
			RefreshButtonStart("tbSatelliteName")

    End Select
End Function

Private Sub LogIt(ByVal txt As String)
	Log(txt, aLog, TIMESTAMPED)
End Sub
